@@ -11,6 +11,7 @@ import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.ruleset.unit.Promotion
 import com.unciv.models.stats.INamed
 import com.unciv.models.stats.Stats
 
@@ -32,6 +33,7 @@ class RulesetValidator(val ruleset: Ruleset) {
 
         val lines = RulesetErrorList()
 
+        /********************** Ruleset Invariant Part **********************/
         // Checks for all mods - only those that can succeed without loading a base ruleset
         // When not checking the entire ruleset, we can only really detect ruleset-invariant errors in uniques
 
@@ -51,17 +53,27 @@ class RulesetValidator(val ruleset: Ruleset) {
 
         for (tech in ruleset.technologies.values) {
             for (otherTech in ruleset.technologies.values) {
-                if (tech != otherTech && otherTech.column == tech.column && otherTech.row == tech.row)
-                    lines += "${tech.name} is in the same row as ${otherTech.name}!"
+                if (tech != otherTech && otherTech.column?.columnNumber == tech.column?.columnNumber && otherTech.row == tech.row)
+                    lines += "${tech.name} is in the same row and column as ${otherTech.name}!"
             }
 
             checkUniques(tech, lines, rulesetInvariant, tryFixUnknownUniques)
         }
 
+        for (techColumn in ruleset.techColumns){
+            if (techColumn.columnNumber < 0)
+                lines+= "Tech Column number ${techColumn.columnNumber} is negative"
+            if (techColumn.buildingCost == -1)
+                lines.add("Tech Column number ${techColumn.columnNumber} has no explicit building cost", RulesetErrorSeverity.Warning)
+            if (techColumn.wonderCost == -1)
+                lines.add("Tech Column number ${techColumn.columnNumber} has no explicit wonder cost", RulesetErrorSeverity.Warning)
+        }
+
         for (building in ruleset.buildings.values) {
-            if (building.requiredTech == null && building.cost == 0 && !building.hasUnique(
+            if (building.requiredTech == null && building.cost == -1 && !building.hasUnique(
                         UniqueType.Unbuildable))
-                lines += "${building.name} is buildable and therefore must either have an explicit cost or reference an existing tech!"
+                lines.add("${building.name} is buildable and therefore should either have an explicit cost or reference an existing tech!",
+                    RulesetErrorSeverity.Warning)
 
             checkUniques(building, lines, rulesetInvariant, tryFixUnknownUniques)
 
@@ -129,6 +141,8 @@ class RulesetValidator(val ruleset: Ruleset) {
         // Quit here when no base ruleset is loaded - references cannot be checked
         if (!ruleset.modOptions.isBaseRuleset) return lines
 
+        /********************** Ruleset Specific Part **********************/
+
         val vanillaRuleset = RulesetCache.getVanillaRuleset()  // for UnitTypes fallback
 
 
@@ -167,8 +181,7 @@ class RulesetValidator(val ruleset: Ruleset) {
                     lines += "${unit.name} contains promotion $promotion which does not exist!"
             if (!ruleset.unitTypes.containsKey(unit.unitType) && (ruleset.unitTypes.isNotEmpty() || !vanillaRuleset.unitTypes.containsKey(unit.unitType)))
                 lines += "${unit.name} is of type ${unit.unitType}, which does not exist!"
-            for (unique in unit.getMatchingUniques(UniqueType.ConstructImprovementConsumingUnit)
-                + unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)) {
+            for (unique in unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)) {
                 val improvementName = unique.params[0]
                 if (ruleset.tileImprovements[improvementName]==null) continue // this will be caught in the checkUniques
                 if ((ruleset.tileImprovements[improvementName] as Stats).none() &&
@@ -376,6 +389,7 @@ class RulesetValidator(val ruleset: Ruleset) {
                 if (!ruleset.unitTypes.containsKey(unitType) && (ruleset.unitTypes.isNotEmpty() || !vanillaRuleset.unitTypes.containsKey(unitType)))
                     lines.add("${promotion.name} references unit type $unitType, which does not exist!", RulesetErrorSeverity.Warning)
             checkUniques(promotion, lines, rulesetSpecific, tryFixUnknownUniques)
+            checkPromotionCircularReferences(lines)
         }
 
         for (unitType in ruleset.unitTypes.values) {
@@ -405,9 +419,8 @@ class RulesetValidator(val ruleset: Ruleset) {
                 val errors = checkUnique(
                     unique,
                     tryFixUnknownUniques,
-                    cityStateType.name,
-                    rulesetSpecific,
-                    UniqueTarget.CityState
+                    cityStateType,
+                    rulesetSpecific
                 )
                 lines.addAll(errors)
             }
@@ -416,6 +429,24 @@ class RulesetValidator(val ruleset: Ruleset) {
         return lines
     }
 
+    private fun checkPromotionCircularReferences(lines: RulesetErrorList) {
+        fun recursiveCheck(history: LinkedHashSet<Promotion>, promotion: Promotion, level: Int) {
+            if (promotion in history) {
+                lines.add("Circular Reference in Promotions: ${history.joinToString("→") { it.name }}→${promotion.name}", RulesetErrorSeverity.Warning)
+                return
+            }
+            if (level > 99) return
+            history.add(promotion)
+            for (prerequisiteName in promotion.prerequisites) {
+                val prerequisite = ruleset.unitPromotions[prerequisiteName] ?: continue
+                recursiveCheck(history.toCollection(linkedSetOf()), prerequisite, level + 1)
+            }
+        }
+        for (promotion in ruleset.unitPromotions.values) {
+            recursiveCheck(linkedSetOf(), promotion, 0)
+        }
+    }
+
 
     fun checkUniques(
         uniqueContainer: IHasUniques,
@@ -423,15 +454,12 @@ class RulesetValidator(val ruleset: Ruleset) {
         severityToReport: UniqueType.UniqueComplianceErrorSeverity,
         tryFixUnknownUniques: Boolean
     ) {
-        val name = if (uniqueContainer is INamed) uniqueContainer.name else ""
-
         for (unique in uniqueContainer.uniqueObjects) {
             val errors = checkUnique(
                 unique,
                 tryFixUnknownUniques,
-                name,
-                severityToReport,
-                uniqueContainer.getUniqueTarget()
+                uniqueContainer as? INamed,
+                severityToReport
             )
             lines.addAll(errors)
         }
@@ -440,10 +468,11 @@ class RulesetValidator(val ruleset: Ruleset) {
     fun checkUnique(
         unique: Unique,
         tryFixUnknownUniques: Boolean,
-        name: String,
-        severityToReport: UniqueType.UniqueComplianceErrorSeverity,
-        uniqueTarget: UniqueTarget
+        namedObj: INamed?,
+        severityToReport: UniqueType.UniqueComplianceErrorSeverity
     ): List<RulesetError> {
+        var name = namedObj?.name ?: ""
+        if (namedObj != null && namedObj is IRulesetObject) name = "${namedObj.originRuleset}: $name"
         if (unique.type == null) {
             if (!tryFixUnknownUniques) return emptyList()
             val similarUniques = UniqueType.values().filter {
@@ -487,14 +516,16 @@ class RulesetValidator(val ruleset: Ruleset) {
 
         val rulesetErrors = RulesetErrorList()
 
+        if (namedObj is IHasUniques && !unique.type.canAcceptUniqueTarget(namedObj.getUniqueTarget()))
+            rulesetErrors.add(RulesetError("$name's unique \"${unique.text}\" is not allowed on its target type", RulesetErrorSeverity.Warning))
+
         val typeComplianceErrors = unique.type.getComplianceErrors(unique, ruleset)
         for (complianceError in typeComplianceErrors) {
-            // TODO: Make this Error eventually, this is Not Good
             if (complianceError.errorSeverity <= severityToReport)
                 rulesetErrors.add(RulesetError("$name's unique \"${unique.text}\" contains parameter ${complianceError.parameterName}," +
                         " which does not fit parameter type" +
                         " ${complianceError.acceptableParameterTypes.joinToString(" or ") { it.parameterName }} !",
-                    RulesetErrorSeverity.Warning
+                    complianceError.errorSeverity.getRulesetErrorSeverity(severityToReport)
                 ))
         }
 
@@ -506,13 +537,20 @@ class RulesetValidator(val ruleset: Ruleset) {
                     RulesetErrorSeverity.Warning
                 )
             } else {
+                if (conditional.type.targetTypes.none { it.modifierType != UniqueTarget.ModifierType.None })
+                    rulesetErrors.add("$name's unique \"${unique.text}\" contains the conditional \"${conditional.text}\"," +
+                        " which is a Unique type not allowed as conditional or trigger.",
+                        RulesetErrorSeverity.Warning)
+
                 val conditionalComplianceErrors =
                         conditional.type.getComplianceErrors(conditional, ruleset)
                 for (complianceError in conditionalComplianceErrors) {
                     if (complianceError.errorSeverity == severityToReport)
-                        rulesetErrors += "$name's unique \"${unique.text}\" contains the conditional \"${conditional.text}\"." +
+                        rulesetErrors.add(RulesetError( "$name's unique \"${unique.text}\" contains the conditional \"${conditional.text}\"." +
                                 " This contains the parameter ${complianceError.parameterName} which does not fit parameter type" +
-                                " ${complianceError.acceptableParameterTypes.joinToString(" or ") { it.parameterName }} !"
+                                " ${complianceError.acceptableParameterTypes.joinToString(" or ") { it.parameterName }} !",
+                            complianceError.errorSeverity.getRulesetErrorSeverity(severityToReport)
+                        ))
                 }
             }
         }
@@ -543,6 +581,7 @@ class RulesetValidator(val ruleset: Ruleset) {
 
 
 class RulesetError(val text:String, val errorSeverityToReport: RulesetErrorSeverity)
+
 enum class RulesetErrorSeverity(val color: Color) {
     OK(Color.GREEN),
     WarningOptionsOnly(Color.YELLOW),
@@ -559,6 +598,23 @@ class RulesetErrorList : ArrayList<RulesetError>() {
         add(RulesetError(text, errorSeverityToReport))
     }
 
+    override fun add(element: RulesetError): Boolean {
+        // Suppress duplicates due to the double run of some checks for invariant/specific,
+        // Without changing collection type or making RulesetError obey the equality contract
+        val existing = firstOrNull { it.text == element.text }
+            ?: return super.add(element)
+        if (existing.errorSeverityToReport >= element.errorSeverityToReport) return false
+        remove(existing)
+        return super.add(element)
+    }
+
+    override fun addAll(elements: Collection<RulesetError>): Boolean {
+        var result = false
+        for (element in elements)
+            if (add(element)) result = true
+        return result
+    }
+
     fun getFinalSeverity(): RulesetErrorSeverity {
         if (isEmpty()) return RulesetErrorSeverity.OK
         return this.maxOf { it.errorSeverityToReport }
@@ -576,5 +632,10 @@ class RulesetErrorList : ArrayList<RulesetError>() {
     fun getErrorText(filter: (RulesetError)->Boolean) =
             filter(filter)
                 .sortedByDescending { it.errorSeverityToReport }
-                .joinToString("\n") { it.errorSeverityToReport.name + ": " + it.text }
+                .joinToString("\n") {
+                    it.errorSeverityToReport.name + ": " +
+                        // This will go through tr(), unavoidably, which will move the conditionals
+                        // out of place. Prevent via kludge:
+                        it.text.replace('<','〈').replace('>','〉')
+                }
 }
