@@ -22,7 +22,7 @@ import com.unciv.ui.components.UncivSlider
 import com.unciv.ui.components.WrappableLabel
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.darken
-import com.unciv.ui.components.extensions.onClick
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.extensions.pad
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
@@ -106,7 +106,7 @@ class MapEditorViewTab(
             naturalWonders.clear()
             tileMap.values.asSequence()
                 .mapNotNull { it.naturalWonder }
-                .sortedWith(compareBy(collator) { it.tr() })
+                .sortedWith(compareBy(collator) { it.tr(hideIcons = true) })
                 .forEach {
                     naturalWonders.add(it, 1)
                 }
@@ -138,7 +138,7 @@ class MapEditorViewTab(
                 startsOutOpened = false,
                 headerPad = 5f
             ) {
-                it.add(MarkupRenderer.render(lines.toList(), iconDisplay = IconDisplay.NoLink) { name ->
+                it.add(MarkupRenderer.render(lines.asIterable(), iconDisplay = IconDisplay.NoLink) { name ->
                     scrollToStartOfNation(name)
                 })
             }).row()
@@ -260,11 +260,11 @@ class MapEditorViewTab(
         startingLocationsByNation.asSequence()
         .filter { tile == null || tile in it.value }
         .mapNotNull { ruleset!!.nations[it.key] }
-        .sortedWith(compareBy<Nation>{ it.isCityState }.thenBy(collator) { it.name.tr() })
+        .sortedWith(compareBy<Nation>{ it.isCityState }.thenBy(collator) { it.name.tr(hideIcons = true) })
 
     private fun TileMap.getStartingLocationSummary() =
         startingLocationsByNation.asSequence()
         .mapNotNull { if (it.key in ruleset!!.nations) ruleset!!.nations[it.key]!! to it.value.size else null }
-        .sortedWith(compareBy<Pair<Nation,Int>>{ it.first.isCityState }.thenBy(collator) { it.first.name.tr() })
+        .sortedWith(compareBy<Pair<Nation,Int>>{ it.first.isCityState }.thenBy(collator) { it.first.name.tr(hideIcons = true) })
         .map { it.first.name to it.second }
 }
